import { Collection } from './collection';
import { ClientMessageAll, ServerMessageAll } from './contract';
import { Subscriptions } from '@deepkit/core-rxjs';
import { Observable, Subscription } from 'rxjs';
import { ClassType } from '@deepkit/core';
import { PropertySchema, Types } from '@deepkit/type';
export interface ConnectionWriterStream {
    send(json: string): Promise<boolean>;
    bufferedAmount(): number;
}
export declare class AlreadyEncoded {
    type: Types;
    value: any;
    constructor(type: Types, value: any);
}
export interface ConnectionWriterInterface {
    write(message: ServerMessageAll): void;
    cancelMessage(id: number): void;
    complete(id: number): void;
    ack(id: number): void;
    sendError(id: number, errorObject: any, code?: string): void;
}
export declare function isObservable(obj: any): obj is Observable<any>;
export declare class SimpleConnectionWriter implements ConnectionWriterInterface {
    write(message: ServerMessageAll): void;
    cancelMessage(messageId: number): void;
    complete(id: number): void;
    ack(id: number): void;
    sendError(id: number, errorObject: any, code?: string): void;
}
export declare class ConnectionWriter extends SimpleConnectionWriter {
    protected socket: ConnectionWriterStream;
    protected chunkIds: number;
    protected cancelSending: {
        [messageId: number]: () => void;
    };
    constructor(socket: ConnectionWriterStream);
    /**
     * This function handles backpressure and waits if necessary
     */
    delayedWrite(data: string): Promise<void>;
    /**
     * Queued writes will be canceled.
     * @param messageId
     */
    cancelMessage(messageId: number): void;
    write(message: ServerMessageAll): Promise<void>;
    complete(id: number): void;
    ack(id: number): void;
    sendError(id: number, errorObject: any, code?: string): void;
}
export declare class ConnectionMiddleware {
    protected collectionSubscriptions: {
        [messageId: string]: Subscriptions;
    };
    protected collections: {
        [messageId: string]: Collection<any>;
    };
    protected subjectSubscriptions: {
        [messageId: string]: Subscriptions;
    };
    protected observables: {
        [messageId: string]: {
            p?: PropertySchema;
            prefix: string;
            observable: Observable<any>;
            subscriber: {
                [subscriberId: string]: Subscription;
            };
        };
    };
    protected entitySent: {
        [messageId: string]: {
            classType: ClassType;
            id: string;
        };
    };
    destroy(): void;
    messageIn(message: ClientMessageAll, writer: ConnectionWriterInterface): Promise<void>;
    actionMessageOut(message: ClientMessageAll, result: any, propertySchema: PropertySchema, //of the method return type
    controllerName: string, actionName: string, writer: ConnectionWriterInterface): Promise<void>;
}
