"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var JobConfig_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Job = exports.JobModelGraph = exports.JobModelNode = exports.JobModelGraphSnapshot = exports.JobModelGraphSnapshotLayer = exports.JobModelGraphSnapshotHistogram = exports.JobTask = exports.JobTaskQueue = exports.JobTaskInstance = exports.PullStats = exports.PullStatsStatus = exports.Channel = exports.JobTaskInstanceStatus = exports.JobTaskStatus = exports.JobStatus = exports.JobDockerImage = exports.JobDocker = exports.JobGit = exports.JobEnvironment = exports.JobEnvironmentPython = exports.JobConfig = exports.JobTaskConfig = exports.JobTaskConfigBase = exports.JobTaskCommand = exports.JobResources = exports.JobConfigDocker = exports.JobAssignedResources = exports.JobAssignedResourcesGpu = void 0;
const type_1 = require("@deepkit/type");
const core_1 = require("@deepkit/core");
let JobAssignedResourcesGpu = class JobAssignedResourcesGpu {
    constructor(uuid, name, memory) {
        this.uuid = uuid;
        this.name = name;
        this.memory = memory;
    }
};
JobAssignedResourcesGpu = __decorate([
    type_1.Entity('JobAssignedResourcesGpu'),
    __param(0, type_1.t.name('uuid')),
    __param(1, type_1.t.name('name')),
    __param(2, type_1.t.name('memory')),
    __metadata("design:paramtypes", [String, String, Number])
], JobAssignedResourcesGpu);
exports.JobAssignedResourcesGpu = JobAssignedResourcesGpu;
let JobAssignedResources = class JobAssignedResources {
    constructor() {
        this.cpu = 0;
        this.memory = 0;
        this.gpus = [];
    }
    getMinGpuMemory() {
        let minGpuMemory = 0;
        for (const gpu of this.gpus) {
            if (!minGpuMemory || gpu.memory < minGpuMemory)
                minGpuMemory = gpu.memory;
        }
        return minGpuMemory;
    }
    getMaxGpuMemory() {
        let maxGpuMemory = 0;
        for (const gpu of this.gpus) {
            if (gpu.memory > maxGpuMemory)
                maxGpuMemory = gpu.memory;
        }
        return maxGpuMemory;
    }
    getGpuUUIDs() {
        return this.gpus.map(v => v.uuid);
    }
    getGpuMemoryRange() {
        const min = this.getMinGpuMemory();
        const max = this.getMaxGpuMemory();
        if (min === max)
            return `${min}`;
        return `${min}-${max}`;
    }
};
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobAssignedResources.prototype, "cpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobAssignedResources.prototype, "memory", void 0);
__decorate([
    type_1.t.array(JobAssignedResourcesGpu),
    __metadata("design:type", Array)
], JobAssignedResources.prototype, "gpus", void 0);
JobAssignedResources = __decorate([
    type_1.Entity('JobAssignedResources')
], JobAssignedResources);
exports.JobAssignedResources = JobAssignedResources;
class JobConfigDocker {
    constructor() {
        this.env = []; //e.g. ["PATH=bla"]
        this.binds = []; //e.g. ["/tmp:/tmp"]
        this.links = []; //e.g. ["redis3:redis"]
    }
}
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobConfigDocker.prototype, "env", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobConfigDocker.prototype, "binds", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobConfigDocker.prototype, "links", void 0);
exports.JobConfigDocker = JobConfigDocker;
class JobResources {
    constructor() {
        this.minCpu = 0;
        this.maxCpu = 0;
        this.cpu = 0;
        this.allMemory = true;
        this.minMemory = 0;
        this.maxMemory = 0;
        this.memory = 0;
        this.minGpu = 0;
        this.maxGpu = 0;
        this.gpu = 0;
        /**
         * Value in GB. Defines minimum gpu memory that is necessary.
         */
        this.minGpuMemory = 0;
    }
    static fromPartial(partial) {
        const resources = new JobResources;
        for (const [i, v] of core_1.eachPair(partial)) {
            resources[i] = v;
        }
        return resources;
    }
    normalizeValues() {
        this.cpu = Math.max(this.cpu, 0);
        this.maxCpu = Math.max(this.maxCpu, 0);
        this.minCpu = Math.max(this.minCpu, 0);
        this.memory = Math.max(this.memory, 0);
        this.maxMemory = Math.max(this.maxMemory, 0);
        this.minMemory = Math.max(this.minMemory, 0);
        this.gpu = Math.max(this.gpu, 0);
        this.maxGpu = Math.max(this.maxGpu, 0);
        this.minGpu = Math.max(this.minGpu, 0);
        this.minGpuMemory = Math.max(this.minGpuMemory, 0);
    }
    getMinCpu() {
        return Math.max(this.minCpu || this.cpu, 1);
    }
    getMaxCpu() {
        return Math.max(this.maxCpu || this.cpu, 1);
    }
    getMinMemory() {
        return Math.max(this.minMemory || this.memory, 1);
    }
    getMaxMemory() {
        return Math.max(this.maxMemory || this.memory, 1);
    }
    getMinGpu() {
        return Math.max(this.minGpu || this.gpu, 0);
    }
    getMaxGpu() {
        return Math.max(this.maxGpu || this.gpu, 0);
    }
}
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "minCpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "maxCpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "cpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Boolean)
], JobResources.prototype, "allMemory", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "minMemory", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "maxMemory", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "memory", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "minGpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "maxGpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "gpu", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobResources.prototype, "minGpuMemory", void 0);
exports.JobResources = JobResources;
let JobTaskCommand = class JobTaskCommand {
    constructor(name = '', command = '') {
        this.name = name;
        this.command = command;
    }
};
JobTaskCommand = __decorate([
    __param(0, type_1.t.name('name')),
    __param(1, type_1.t.name('command')),
    __metadata("design:paramtypes", [String, String])
], JobTaskCommand);
exports.JobTaskCommand = JobTaskCommand;
class JobTaskConfigBase {
    constructor() {
        this.label = '';
        this.install = [];
        this.dockerfile = '';
        this.install_files = [];
        this.image = '';
        this.output = [];
        this.environmentVariables = [];
        this.nodeIds = [];
        this.nodes = [];
        this.clusters = [];
        this.commands = [];
        this.args = [];
        this.resources = new JobResources;
        this.docker = new JobConfigDocker;
    }
    isDockerImage() {
        return !!this.image;
    }
    hasCommand() {
        return this.commands.length > 0;
    }
    get installString() {
        return this.install.join('\n');
    }
    set installString(v) {
        this.install = v.split('\n');
    }
    set installFilesString(v) {
        this.install_files = v.split('\n');
    }
    get installFilesString() {
        return this.install_files.join('\n');
    }
}
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobTaskConfigBase.prototype, "label", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "install", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobTaskConfigBase.prototype, "dockerfile", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "install_files", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobTaskConfigBase.prototype, "image", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "output", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "environmentVariables", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "nodeIds", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "nodes", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "clusters", void 0);
__decorate([
    type_1.t.array(JobTaskCommand),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "commands", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfigBase.prototype, "args", void 0);
__decorate([
    type_1.t.type(JobResources),
    __metadata("design:type", JobResources)
], JobTaskConfigBase.prototype, "resources", void 0);
__decorate([
    type_1.t.type(JobConfigDocker),
    __metadata("design:type", JobConfigDocker)
], JobTaskConfigBase.prototype, "docker", void 0);
exports.JobTaskConfigBase = JobTaskConfigBase;
class JobTaskConfig extends JobTaskConfigBase {
    constructor() {
        super(...arguments);
        /**
         * Will be set by config loader.
         */
        this.name = '';
        this.replicas = 1;
        this.depends_on = [];
        /**
         * Will be set config loader
         */
        this.configValuesNoOverwrite = {};
    }
    isRoot() {
        return this.depends_on.length === 0;
    }
}
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobTaskConfig.prototype, "name", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobTaskConfig.prototype, "replicas", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobTaskConfig.prototype, "depends_on", void 0);
__decorate([
    type_1.t.map(Boolean),
    __metadata("design:type", Object)
], JobTaskConfig.prototype, "configValuesNoOverwrite", void 0);
exports.JobTaskConfig = JobTaskConfig;
let JobConfig = JobConfig_1 = class JobConfig extends JobTaskConfigBase {
    constructor() {
        super(...arguments);
        this.parameters = {};
        this.path = '';
        this.ignore = [];
        this.priority = 0;
        this.import = '';
        this.tasks = {};
        this.resolved = false;
    }
    getLabel() {
        if (!this.label && this.path) {
            let path = this.path;
            if (path.endsWith('.yaml'))
                path = path.substr(0, path.length - 5);
            if (path.endsWith('.yml'))
                path = path.substr(0, path.length - 4);
            const index = path.lastIndexOf('/');
            if (index !== -1) {
                path = path.substr(index + 1);
            }
            if (path === 'deepkit') {
                return 'Default';
            }
            return path;
        }
        return this.label;
    }
    /**
     * Writes all values of the root config into task config and the values from the original config (configValuesToOverwrite)
     * file to task config (so user can override them)
     */
    resolveInheritance() {
        for (const task of core_1.each(this.getTasks())) {
            for (const name of JobConfig_1.inheritTaskProperties) {
                if (task.configValuesNoOverwrite[name])
                    continue;
                task[name] = this[name];
            }
        }
    }
    getTasks() {
        if (Object.keys(this.tasks).length === 0) {
            this.tasks = { main: new JobTaskConfig };
            this.tasks.main.name = 'main';
        }
        if (!this.resolved) {
            this.resolved = true;
            this.resolveInheritance();
        }
        return this.tasks;
    }
};
JobConfig.inheritTaskProperties = [
    'install',
    'dockerfile',
    'install_files',
    'image',
    'environmentVariables',
    'clusters',
    'nodes',
    'nodeIds',
    'resources',
    'commands',
    'args',
    'output',
    'docker'
];
__decorate([
    type_1.t.map(type_1.t.any),
    __metadata("design:type", Object)
], JobConfig.prototype, "parameters", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobConfig.prototype, "path", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobConfig.prototype, "ignore", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobConfig.prototype, "priority", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobConfig.prototype, "import", void 0);
__decorate([
    type_1.t.map(JobTaskConfig),
    __metadata("design:type", Object)
], JobConfig.prototype, "tasks", void 0);
JobConfig = JobConfig_1 = __decorate([
    type_1.Entity('job-config')
], JobConfig);
exports.JobConfig = JobConfig;
class JobEnvironmentPython {
    constructor() {
        this.pipPackages = {};
    }
}
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironmentPython.prototype, "version", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironmentPython.prototype, "binary", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironmentPython.prototype, "sdkVersion", void 0);
__decorate([
    type_1.t.map(String),
    __metadata("design:type", Object)
], JobEnvironmentPython.prototype, "pipPackages", void 0);
exports.JobEnvironmentPython = JobEnvironmentPython;
class JobEnvironment {
}
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironment.prototype, "hostname", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironment.prototype, "username", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironment.prototype, "platform", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironment.prototype, "release", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironment.prototype, "arch", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Number)
], JobEnvironment.prototype, "uptime", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobEnvironment.prototype, "nodeVersion", void 0);
__decorate([
    type_1.t.map(String).optional,
    __metadata("design:type", Object)
], JobEnvironment.prototype, "environmentVariables", void 0);
__decorate([
    type_1.t.type(JobEnvironmentPython).optional,
    __metadata("design:type", JobEnvironmentPython)
], JobEnvironment.prototype, "python", void 0);
exports.JobEnvironment = JobEnvironment;
let JobGit = class JobGit {
    constructor(commit, message, authorEmail, date) {
        this.commit = commit;
        this.message = message;
        this.authorEmail = authorEmail;
        this.date = date;
    }
};
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobGit.prototype, "author", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobGit.prototype, "branch", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobGit.prototype, "origin", void 0);
JobGit = __decorate([
    __param(0, type_1.t.name('commit')),
    __param(1, type_1.t.name('message')),
    __param(2, type_1.t.name('authorEmail').optional),
    __param(3, type_1.t.name('date').optional),
    __metadata("design:paramtypes", [String, String, String, Date])
], JobGit);
exports.JobGit = JobGit;
class JobDocker {
}
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobDocker.prototype, "runOnVersion", void 0);
exports.JobDocker = JobDocker;
class JobDockerImage {
}
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobDockerImage.prototype, "name", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobDockerImage.prototype, "id", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Number)
], JobDockerImage.prototype, "size", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobDockerImage.prototype, "os", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobDockerImage.prototype, "arch", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobDockerImage.prototype, "created", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobDockerImage.prototype, "builtWithDockerVersion", void 0);
exports.JobDockerImage = JobDockerImage;
//note: these codes are hardcoded in the SDKs as well
var JobStatus;
(function (JobStatus) {
    JobStatus[JobStatus["creating"] = 0] = "creating";
    JobStatus[JobStatus["created"] = 50] = "created";
    JobStatus[JobStatus["running"] = 100] = "running";
    JobStatus[JobStatus["done"] = 150] = "done";
    JobStatus[JobStatus["aborted"] = 200] = "aborted";
    JobStatus[JobStatus["failed"] = 250] = "failed";
    JobStatus[JobStatus["crashed"] = 300] = "crashed";
})(JobStatus = exports.JobStatus || (exports.JobStatus = {}));
//note: these codes are hardcoded in the SDKs as well
var JobTaskStatus;
(function (JobTaskStatus) {
    JobTaskStatus[JobTaskStatus["pending"] = 0] = "pending";
    JobTaskStatus[JobTaskStatus["queued"] = 100] = "queued";
    JobTaskStatus[JobTaskStatus["assigned"] = 150] = "assigned";
    JobTaskStatus[JobTaskStatus["started"] = 300] = "started";
    //beginning with that ended
    JobTaskStatus[JobTaskStatus["done"] = 500] = "done";
    JobTaskStatus[JobTaskStatus["aborted"] = 550] = "aborted";
    JobTaskStatus[JobTaskStatus["failed"] = 600] = "failed";
    JobTaskStatus[JobTaskStatus["crashed"] = 650] = "crashed";
})(JobTaskStatus = exports.JobTaskStatus || (exports.JobTaskStatus = {}));
//note: these codes are hardcoded in the SDKs as well
var JobTaskInstanceStatus;
(function (JobTaskInstanceStatus) {
    JobTaskInstanceStatus[JobTaskInstanceStatus["pending"] = 0] = "pending";
    JobTaskInstanceStatus[JobTaskInstanceStatus["booting"] = 200] = "booting";
    JobTaskInstanceStatus[JobTaskInstanceStatus["docker_pull"] = 220] = "docker_pull";
    JobTaskInstanceStatus[JobTaskInstanceStatus["docker_build_await"] = 230] = "docker_build_await";
    JobTaskInstanceStatus[JobTaskInstanceStatus["docker_build"] = 235] = "docker_build";
    JobTaskInstanceStatus[JobTaskInstanceStatus["joining_network"] = 250] = "joining_network";
    JobTaskInstanceStatus[JobTaskInstanceStatus["checkout_files"] = 260] = "checkout_files";
    JobTaskInstanceStatus[JobTaskInstanceStatus["started"] = 300] = "started";
    //beginning with that ended
    JobTaskInstanceStatus[JobTaskInstanceStatus["done"] = 500] = "done";
    JobTaskInstanceStatus[JobTaskInstanceStatus["aborted"] = 550] = "aborted";
    JobTaskInstanceStatus[JobTaskInstanceStatus["failed"] = 600] = "failed";
    JobTaskInstanceStatus[JobTaskInstanceStatus["crashed"] = 650] = "crashed";
})(JobTaskInstanceStatus = exports.JobTaskInstanceStatus || (exports.JobTaskInstanceStatus = {}));
let Channel = class Channel {
    constructor() {
        /**
         * This might be empty.
         */
        this.traces = [];
        this.kpiTrace = 0;
        this.maxOptimization = true;
        this.lastValue = [];
    }
};
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], Channel.prototype, "traces", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Boolean)
], Channel.prototype, "kpi", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Channel.prototype, "kpiTrace", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Boolean)
], Channel.prototype, "maxOptimization", void 0);
__decorate([
    type_1.t.array(type_1.t.any),
    __metadata("design:type", Array)
], Channel.prototype, "lastValue", void 0);
__decorate([
    type_1.t.any.optional,
    __metadata("design:type", Object)
], Channel.prototype, "xaxis", void 0);
__decorate([
    type_1.t.any.optional,
    __metadata("design:type", Object)
], Channel.prototype, "yaxis", void 0);
__decorate([
    type_1.t.any.optional,
    __metadata("design:type", Object)
], Channel.prototype, "layout", void 0);
Channel = __decorate([
    type_1.Entity('job/channel')
], Channel);
exports.Channel = Channel;
var PullStatsStatus;
(function (PullStatsStatus) {
    PullStatsStatus["waiting"] = "waiting";
    PullStatsStatus["downloading"] = "downloading";
    PullStatsStatus["extracting"] = "extracting";
    PullStatsStatus["verifying"] = "verifying";
    PullStatsStatus["done"] = "done";
})(PullStatsStatus = exports.PullStatsStatus || (exports.PullStatsStatus = {}));
let PullStats = class PullStats {
    constructor(id) {
        this.id = id;
        this.current = 0;
        this.total = 0;
        this.status = PullStatsStatus.waiting;
    }
};
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], PullStats.prototype, "current", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], PullStats.prototype, "total", void 0);
__decorate([
    type_1.t.enum(PullStatsStatus),
    __metadata("design:type", String)
], PullStats.prototype, "status", void 0);
PullStats = __decorate([
    __param(0, type_1.t.name('id')),
    __metadata("design:paramtypes", [String])
], PullStats);
exports.PullStats = PullStats;
let JobTaskInstance = class JobTaskInstance {
    constructor(id) {
        this.id = id;
        this.status = JobTaskInstanceStatus.pending;
        this.uploadOutputCurrent = -1;
        this.uploadOutputTotal = 0;
        this.environment = new JobEnvironment;
        this.docker = new JobDocker;
        this.dockerPullStats = {};
        this.dockerImage = new JobDockerImage;
        this.error = '';
        this.assignedResources = new JobAssignedResources;
    }
    getOrCreatePullStats(id) {
        if (!this.dockerPullStats[id]) {
            this.dockerPullStats[id] = new PullStats(id);
        }
        return this.dockerPullStats[id];
    }
    isRunning() {
        return this.isStarted() && !this.isEnded();
    }
    isStarted() {
        return this.status >= JobTaskInstanceStatus.booting;
    }
    isEnded() {
        return this.status >= JobTaskInstanceStatus.done;
    }
    isDockerPull() {
        return this.status === JobTaskInstanceStatus.docker_pull;
    }
    elapsedTime() {
        if (this.ended && this.started) {
            return (this.ended.getTime() - this.started.getTime()) / 1000;
        }
        if (this.started) {
            return ((new Date).getTime() - this.started.getTime()) / 1000;
        }
        return undefined;
    }
};
__decorate([
    type_1.t.enum(JobTaskInstanceStatus),
    __metadata("design:type", Number)
], JobTaskInstance.prototype, "status", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobTaskInstance.prototype, "uploadOutputCurrent", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobTaskInstance.prototype, "uploadOutputTotal", void 0);
__decorate([
    type_1.t.type(JobEnvironment),
    __metadata("design:type", JobEnvironment)
], JobTaskInstance.prototype, "environment", void 0);
__decorate([
    type_1.t.type(JobDocker),
    __metadata("design:type", JobDocker)
], JobTaskInstance.prototype, "docker", void 0);
__decorate([
    type_1.t.map(PullStats),
    __metadata("design:type", Object)
], JobTaskInstance.prototype, "dockerPullStats", void 0);
__decorate([
    type_1.t.type(JobDockerImage),
    __metadata("design:type", JobDockerImage)
], JobTaskInstance.prototype, "dockerImage", void 0);
__decorate([
    type_1.t.uuid.optional,
    __metadata("design:type", String)
], JobTaskInstance.prototype, "node", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Number)
], JobTaskInstance.prototype, "exitCode", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobTaskInstance.prototype, "error", void 0);
__decorate([
    type_1.t.type(JobAssignedResources),
    __metadata("design:type", JobAssignedResources)
], JobTaskInstance.prototype, "assignedResources", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobTaskInstance.prototype, "started", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobTaskInstance.prototype, "ended", void 0);
JobTaskInstance = __decorate([
    __param(0, type_1.t.name('id')),
    __metadata("design:paramtypes", [Number])
], JobTaskInstance);
exports.JobTaskInstance = JobTaskInstance;
class JobTaskQueue {
    constructor() {
        this.position = 0;
        this.tries = 0;
        this.result = '';
        this.added = new Date();
    }
}
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobTaskQueue.prototype, "position", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobTaskQueue.prototype, "tries", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobTaskQueue.prototype, "result", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Date)
], JobTaskQueue.prototype, "added", void 0);
exports.JobTaskQueue = JobTaskQueue;
let JobTask = class JobTask {
    constructor(name, replicas) {
        this.name = name;
        this.queue = new JobTaskQueue;
        this.status = JobTaskStatus.pending;
        this.instances = [];
        for (let i = 0; i < replicas; i++) {
            this.instances[i] = new JobTaskInstance(i);
        }
    }
    elapsedTime() {
        if (this.ended && this.started) {
            return (this.ended.getTime() - this.started.getTime()) / 1000;
        }
        if (this.started) {
            return ((new Date).getTime() - this.started.getTime()) / 1000;
        }
        return undefined;
    }
    getInstances() {
        return this.instances;
    }
    getRunningInstances() {
        const result = [];
        for (const instance of this.instances) {
            if (instance.isRunning())
                result.push(instance);
        }
        return result;
    }
    getFirstInstance() {
        return this.instances[0];
    }
    isErrored() {
        return this.status === JobTaskStatus.crashed
            || this.status === JobTaskStatus.failed
            || this.status === JobTaskStatus.aborted;
    }
    getInstance(replica) {
        if (!this.instances[replica]) {
            throw new Error(`Replica #${replica} of task ${this.name} does not exist.`);
        }
        return this.instances[replica];
    }
    isStarted() {
        return this.status >= JobTaskStatus.started;
    }
    isQueued() {
        return this.status === JobTaskStatus.queued;
    }
    isRunning() {
        return this.isStarted() && !this.isEnded();
    }
    isEnded() {
        return this.status >= JobTaskStatus.done;
    }
    areAllInstancesEnded() {
        return this.instances.every((instance) => {
            return instance.isEnded();
        });
    }
    calculateStatusByInstances() {
        let status = JobTaskStatus.done;
        for (const instance of this.instances) {
            if (status === JobTaskStatus.done) {
                //allowed to set to all
                if (instance.status === JobTaskInstanceStatus.aborted) {
                    status = JobTaskStatus.aborted;
                }
                if (instance.status === JobTaskInstanceStatus.failed) {
                    status = JobTaskStatus.failed;
                }
                if (instance.status === JobTaskInstanceStatus.crashed) {
                    status = JobTaskStatus.crashed;
                }
            }
            if (status === JobTaskStatus.aborted) {
                if (instance.status === JobTaskInstanceStatus.failed) {
                    status = JobTaskStatus.failed;
                }
                if (instance.status === JobTaskInstanceStatus.crashed) {
                    status = JobTaskStatus.crashed;
                }
            }
        }
        return status;
    }
};
__decorate([
    type_1.t.type(JobTaskQueue),
    __metadata("design:type", JobTaskQueue)
], JobTask.prototype, "queue", void 0);
__decorate([
    type_1.t.enum(JobTaskStatus),
    __metadata("design:type", Number)
], JobTask.prototype, "status", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobTask.prototype, "assigned", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobTask.prototype, "started", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobTask.prototype, "ended", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Number)
], JobTask.prototype, "exitCode", void 0);
__decorate([
    type_1.t.array(JobTaskInstance),
    __metadata("design:type", Array)
], JobTask.prototype, "instances", void 0);
JobTask = __decorate([
    __param(0, type_1.t.name('name')),
    __metadata("design:paramtypes", [String, Number])
], JobTask);
exports.JobTask = JobTask;
let JobModelGraphSnapshotHistogram = class JobModelGraphSnapshotHistogram {
};
__decorate([
    type_1.t.array(Number),
    __metadata("design:type", Array)
], JobModelGraphSnapshotHistogram.prototype, "x", void 0);
__decorate([
    type_1.t.array(Number),
    __metadata("design:type", Array)
], JobModelGraphSnapshotHistogram.prototype, "y", void 0);
JobModelGraphSnapshotHistogram = __decorate([
    type_1.Entity('jobModelGraphSnapshot/histogram')
], JobModelGraphSnapshotHistogram);
exports.JobModelGraphSnapshotHistogram = JobModelGraphSnapshotHistogram;
let JobModelGraphSnapshotLayer = class JobModelGraphSnapshotLayer {
};
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], JobModelGraphSnapshotLayer.prototype, "saved", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobModelGraphSnapshotLayer.prototype, "outputFileId", void 0);
__decorate([
    type_1.t.type(JobModelGraphSnapshotHistogram).optional,
    __metadata("design:type", JobModelGraphSnapshotHistogram)
], JobModelGraphSnapshotLayer.prototype, "weights", void 0);
__decorate([
    type_1.t.type(JobModelGraphSnapshotHistogram).optional,
    __metadata("design:type", JobModelGraphSnapshotHistogram)
], JobModelGraphSnapshotLayer.prototype, "biases", void 0);
JobModelGraphSnapshotLayer = __decorate([
    type_1.Entity('jobModelGraphSnapshot/layer')
], JobModelGraphSnapshotLayer);
exports.JobModelGraphSnapshotLayer = JobModelGraphSnapshotLayer;
let JobModelGraphSnapshot = class JobModelGraphSnapshot {
    constructor(job) {
        this.job = job;
        this.id = type_1.uuid();
        this.version = 0;
        this.created = new Date;
        this.iteration = 0;
        this.live = false;
        this.layerInfo = {};
    }
};
__decorate([
    type_1.t.uuid,
    __metadata("design:type", String)
], JobModelGraphSnapshot.prototype, "id", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobModelGraphSnapshot.prototype, "version", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Date)
], JobModelGraphSnapshot.prototype, "created", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], JobModelGraphSnapshot.prototype, "iteration", void 0);
__decorate([
    type_1.t.index(),
    __metadata("design:type", Boolean)
], JobModelGraphSnapshot.prototype, "live", void 0);
__decorate([
    type_1.t.map(JobModelGraphSnapshotLayer),
    __metadata("design:type", Object)
], JobModelGraphSnapshot.prototype, "layerInfo", void 0);
JobModelGraphSnapshot = __decorate([
    type_1.Entity('jobModelGraphSnapshot'),
    __param(0, type_1.t.uuid.index().name('job')),
    __metadata("design:paramtypes", [String])
], JobModelGraphSnapshot);
exports.JobModelGraphSnapshot = JobModelGraphSnapshot;
class JobModelNode {
    constructor() {
        this.type = ''; //op, dense, conv, concat, reshape, etc.
        this.shape = [];
        this.attributes = {};
        /**
         * input ids from other nodes.
         */
        this.input = [];
        /**
         * input ids from other nodes.
         */
        this.children = [];
    }
    hasSelfActivation() {
        return this.attributes['activation'] && this.attributes['activation'] !== 'linear';
    }
    isSmallBox() {
        return this.type === 'BatchNormalization' || this.type === 'Dropout' || this.type.includes('Padding') || this.type.includes('Pooling');
    }
    hasBorder() {
        return !this.isSmallBox();
    }
    isBorderRadius() {
        return this.type === 'Dense';
    }
    isActivation() {
        return this.type === 'Activation';
    }
}
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobModelNode.prototype, "id", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], JobModelNode.prototype, "label", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobModelNode.prototype, "type", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], JobModelNode.prototype, "op", void 0);
__decorate([
    type_1.t.array(Number),
    __metadata("design:type", Array)
], JobModelNode.prototype, "shape", void 0);
__decorate([
    type_1.t.map(type_1.t.any),
    __metadata("design:type", Object)
], JobModelNode.prototype, "attributes", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobModelNode.prototype, "input", void 0);
__decorate([
    type_1.t.array(JobModelNode),
    __metadata("design:type", Array)
], JobModelNode.prototype, "children", void 0);
exports.JobModelNode = JobModelNode;
let JobModelGraph = class JobModelGraph {
    constructor() {
        this.nodes = [];
        this.inputs = [];
        this.outputs = [];
    }
};
__decorate([
    type_1.t.array(JobModelNode),
    __metadata("design:type", Array)
], JobModelGraph.prototype, "nodes", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobModelGraph.prototype, "inputs", void 0);
__decorate([
    type_1.t.array(String),
    __metadata("design:type", Array)
], JobModelGraph.prototype, "outputs", void 0);
JobModelGraph = __decorate([
    type_1.Entity('job/model/graph')
], JobModelGraph);
exports.JobModelGraph = JobModelGraph;
let Job = class Job {
    constructor(id, project) {
        this.id = id;
        this.project = project;
        this.accessToken = type_1.uuid();
        this.number = 0;
        this.version = 1;
        this.description = '';
        this.connections = 0;
        this.created = new Date();
        this.updated = new Date();
        this.config = new JobConfig;
        /**
         * Whether the job is executed directly in script without Deepkit CLI tools.
         */
        this.selfExecution = false;
        this.hasModelGraph = false;
        this.status = JobStatus.creating;
        this.title = '';
        this.infos = {};
        this.tasks = {};
        //todo, move the content to a file
        this.debugSnapshots = {};
        this.debugActiveLayerWatching = {};
        this.runOnCluster = false;
        this.tags = [];
        //aka epochs
        this.iteration = 0;
        this.iterations = 0;
        this.secondsPerIteration = 0;
        //aka batches
        this.step = 0;
        this.steps = 0;
        this.stepLabel = 'step';
        /**
         * ETA in seconds. Time left.
         */
        this.eta = 0;
        this.speed = 0;
        this.speedLabel = 'sample/s';
        this.channels = {};
    }
};
__decorate([
    type_1.t.uuid.exclude('json'),
    __metadata("design:type", String)
], Job.prototype, "accessToken", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "number", void 0);
__decorate([
    type_1.t.index(),
    __metadata("design:type", Number)
], Job.prototype, "version", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], Job.prototype, "description", void 0);
__decorate([
    type_1.t.uuid.optional,
    __metadata("design:type", String)
], Job.prototype, "cluster", void 0);
__decorate([
    type_1.t.index(),
    __metadata("design:type", Number)
], Job.prototype, "connections", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Date)
], Job.prototype, "created", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Date)
], Job.prototype, "updated", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], Job.prototype, "author", void 0);
__decorate([
    type_1.t.type(JobConfig),
    __metadata("design:type", JobConfig)
], Job.prototype, "config", void 0);
__decorate([
    type_1.t.type(JobGit).optional,
    __metadata("design:type", JobGit)
], Job.prototype, "git", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", String)
], Job.prototype, "configFile", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Boolean)
], Job.prototype, "selfExecution", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Boolean)
], Job.prototype, "hasModelGraph", void 0);
__decorate([
    type_1.t.uuid.optional,
    __metadata("design:type", String)
], Job.prototype, "liveSnapshotId", void 0);
__decorate([
    type_1.t.enum(JobStatus),
    __metadata("design:type", Number)
], Job.prototype, "status", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], Job.prototype, "title", void 0);
__decorate([
    type_1.t.map(type_1.t.any),
    __metadata("design:type", Object)
], Job.prototype, "infos", void 0);
__decorate([
    type_1.t.map(JobTask),
    __metadata("design:type", Object)
], Job.prototype, "tasks", void 0);
__decorate([
    type_1.t.map(type_1.t.any),
    __metadata("design:type", Object)
], Job.prototype, "debugSnapshots", void 0);
__decorate([
    type_1.t.map(Boolean),
    __metadata("design:type", Object)
], Job.prototype, "debugActiveLayerWatching", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Boolean)
], Job.prototype, "runOnCluster", void 0);
__decorate([
    type_1.t.array(String).index(),
    __metadata("design:type", Array)
], Job.prototype, "tags", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], Job.prototype, "assigned", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], Job.prototype, "started", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], Job.prototype, "ended", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], Job.prototype, "stopRequested", void 0);
__decorate([
    type_1.t.optional,
    __metadata("design:type", Date)
], Job.prototype, "ping", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "iteration", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "iterations", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "secondsPerIteration", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "step", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "steps", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], Job.prototype, "stepLabel", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "eta", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", Number)
], Job.prototype, "speed", void 0);
__decorate([
    type_1.t,
    __metadata("design:type", String)
], Job.prototype, "speedLabel", void 0);
__decorate([
    type_1.t.map(Channel),
    __metadata("design:type", Object)
], Job.prototype, "channels", void 0);
Job = __decorate([
    type_1.Entity('job', 'jobs'),
    __param(0, type_1.t.uuid.primary.name('id')),
    __param(1, type_1.t.uuid.index().name('project')),
    __metadata("design:paramtypes", [String, String])
], Job);
exports.Job = Job;
//# sourceMappingURL=entities.js.map